% main_YaleB : Main script for recognition experiments on Yale database using IGO-PCA
%
% Described in 
% G. Tzimiropoulos, S. Zafeiriou and M. Pantic, "Subspace Learning From Image Gradient Orientations", to appear in TPAMI.
% Intelligent Behaviour Understanding Group (IBUG), Department of Computing, Imperial College London
% Version: 1.0, 18/01/2012

clear; clc; close all

load myYaleB
% YaleB contains 64x64 face images taken from the Extended YaleB database
% labels contains the ground truth labels for each of the faces
% train_ind and test_ind are the indices corresponding to a random splitting of YaleB 
% to training (5 images per subject) and testing data

% get training and testing data
train_data = YaleB(:, :, train_ind);
test_data = YaleB(:, :, test_ind);
% get training and testing labels
train_labels = labels(train_ind);
test_labels  = labels(test_ind);

% Train IGO-PCA and IGO-LDA
% train_features are the embeddings of the training data
[IGO_PC train_PCA_features] = IGO_PCA_Train(train_data);
[IGO_FF train_LDA_features] = IGO_LDA_Train(train_data, train_labels);

% Embedding of new samples
% test_features are the embeddings of the testing data
test_PCA_features = IGO_Embedd(test_data, IGO_PC);
test_LDA_features = IGO_Embedd(test_data, IGO_FF);

% Nearest Neigbour Classifier for IGO-PCA
train_features = train_PCA_features;
test_features = test_PCA_features;
correct_PCA = zeros(1, size(test_PCA_features, 1));
for ii = 1:size(test_PCA_features, 1)
    sample = test_PCA_features(1:ii, :).';
    training = train_PCA_features(1:ii, :).';
    distance = 'cosine';
    k = 1;
    class = knnclassify([real(sample) imag(sample)], [real(training) imag(training)], train_labels, k, distance);
    correct_PCA(ii) = length(find(class-test_labels == 0));
end
% recognition rate for IGO-PCA
correct_PCA = correct_PCA./length(test_labels);


% Nearest Neigbour Classifier for IGO-LDA
correct_LDA = zeros(1, size(test_LDA_features, 1));
for ii = 1:size(test_LDA_features, 1)
    sample = test_LDA_features(1:ii, :).';
    training = train_LDA_features(1:ii, :).';
    distance = 'cosine';
    k = 1;
    class = knnclassify([real(sample) imag(sample)], [real(training) imag(training)], train_labels, k, distance);
    correct_LDA(ii) = length(find(class-test_labels == 0));
end
% recognition rate for IGO-LDA
correct_LDA = correct_LDA./length(test_labels);


% plot results
figure; plot(1:length(correct_PCA), correct_PCA, 'red', 1:length(correct_LDA), correct_LDA, 'blue', 'MarkerSize',11, 'linewidth', 2); grid on
set(gca, 'FontSize', 0.1)
set(gca, 'FontWeight', 'bold')
legend('IGO-PCA', 'IGO-LDA')
ylabel('Recognition Rate', 'Interpreter','tex', 'fontsize', 15)
xlabel('Number of Features', 'Interpreter','tex', 'fontsize', 15)






